@@ -2,6 +2,10 @@
 #define __BACKPORT_IF_ETHER_H
 #include_next <linux/if_ether.h>
 
+#ifndef ETH_TLEN
+#define ETH_TLEN	2		/* Octets in ethernet type field */
+#endif
+
 /* See commit b62faf3c in next-20140311 */
 #ifndef ETH_P_80221
 #define ETH_P_80221	0x8917	/* IEEE 802.21 Media Independent Handover Protocol */
