@@ -187,7 +187,8 @@ static void qtnf_netdev_tx_timeout(struct net_device *ndev, unsigned int txqueue
 #if LINUX_VERSION_IS_LESS(5,6,0)
 /* Just declare it here to keep sparse happy */
 void bp_qtnf_netdev_tx_timeout(struct net_device *dev);
-void bp_qtnf_netdev_tx_timeout(struct net_device *dev) {
+void bp_qtnf_netdev_tx_timeout(struct net_device *dev)
+{
 	qtnf_netdev_tx_timeout(dev, 0);
 }
 EXPORT_SYMBOL_GPL(bp_qtnf_netdev_tx_timeout);
